//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include "GuiBBox.h"
#include "file.h"
#include <iostream>
#include <time.h>
#include <stdlib.h>
#include "DrawFileVisitor.h"
#include "DrawFileParser.h"


int DrawFileParser::accept(DrawFileVisitor& visitor)
{
  visitor.fileHeader(0);
  iterate(visitor,sizeof(DrawFileVisitor::FileHeader),getSize());
  return 0;
}

//*********************************************************************

int DrawFileParser::iterate(DrawFileVisitor& visitor, size_t start,size_t end)
{
  while (start<end)
  {
    DrawFileVisitor::ObjectHeader* data=(DrawFileVisitor::ObjectHeader*)getObject(start);
    size_t size=data->size;
    size_t next=start+size;
    switch (data->objectType)
    {
       case 12: if (!visitor.transformedTextObject(start)) return 0;
                break;
       case 2:  if (!visitor.pathObject(start)) return 0;
                break;
       case 13: if (!visitor.transformedSpriteObject(start)) return 0;
                break;
       case 1:  if (!visitor.textObject(start)) return 0;
                break;
       case 5:  if (!visitor.spriteObject(start)) return 0;
                break;
       case 6:  if (!visitor.enteringGroupObject(start)) return 0;
                if (!iterate(visitor,start+sizeof(DrawFileVisitor::GroupObject),next)||
                    !visitor.leavingGroupObject(start)) return 0;
                break;
       case 7:  if (!visitor.taggedObject(start)) return 0;
                break;
       case 0:  if (!visitor.fontTable(start)) return 0;
                break;
       case 9:  if (!visitor.textAreaObject(start)) return 0;
                break;
       case 10: if (!visitor.textColumnObject(start)) return 0;
                break;
       case 11: if (!visitor.optionsObject(start)) return 0;
                break;
      default:  if (!visitor.unknownObject(start)) return 0;
    }
    start=next;
  };
  return 1;
};  

