//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DrawFileVisitor_h
#define DrawFileVisitor_h

#include "GuiBBox.h"
#include <stdlib.h>

class DrawFileVisitor
{ 

  public:
    virtual int fileHeader(size_t start)                      {return start==start;}
    virtual int textObject(size_t start)                      {return start==start;}
    virtual int transformedTextObject(size_t start)           {return start==start;}
    virtual int pathObject(size_t start)                      {return start==start;}
    virtual int spriteObject(size_t start)                    {return start==start;}
    virtual int transformedSpriteObject(size_t start)         {return start==start;}
    virtual int enteringGroupObject(size_t start)             {return start==start;}
    virtual int leavingGroupObject(size_t start)              {return start==start;}
    virtual int taggedObject(size_t start)                    {return start==start;}
    virtual int fontTable(size_t start)                       {return start==start;}
    virtual int textAreaObject(size_t start)                  {return start==start;}
    virtual int textColumnObject(size_t start)                {return start==start;}
    virtual int optionsObject(size_t start)                   {return start==start;}
    virtual int unknownObject(size_t start)                   {return start==start;}

// structures
    class FileHeader
    {
      public:
        char    formatId[4];
        int     majorFormatVersion;
        int     minorFormatVersion;
        char    programId[12];
        GuiBBox bounds;
    };

    class Transform
    {
       public:
       int m0,m1;
       int m2,m3;
       int m4,m5;
    };

    class ObjectHeader
    { 
      public:     
        unsigned int objectType;
        size_t size;
    };

    class BoundedObjectHeader : public ObjectHeader
    {
      public:
         GuiBBox bounds;
    };

    class GroupObject : public BoundedObjectHeader
    {
      public:
        char name[12];
    };

    class FontTable : public ObjectHeader
    {
    };

    class TransformedTextObject : public BoundedObjectHeader
    {
      public:
        Transform matrix;
        enum {KERN = (1<<0),RIGHT_TO_LEFT=(1<<1)};
        int           flags;
        unsigned int  foreColour;
        unsigned int  backColour;
        int           font; //as passed to GuiDrawFileFonts
        size_t        fontXSize;
        size_t        fontYSize;
        int           x;
        int           y;
        char*         getText()  {return sizeof(TransformedTextObject)+(char*)this;}
    };

    class PathObject : public BoundedObjectHeader
    {
      public:
        unsigned int  fillColour;
        unsigned int  outlineColour;
        size_t        outlineWidth;
        unsigned char pathStyle;
        unsigned char reserved;
        unsigned char capWidth;
        unsigned char capHeight;
        bool hasDash()        {return pathStyle & (1<<7);}
        int& value(int n)     {return ((int*)(sizeof(PathObject)+(char*)this))[n];}
        int* dashStart()      {return hasDash()? &value(0):0;}
        int* pathStart()      {return hasDash()? &value(0)+2+value(1):&value(0);}
    };

    class TransformedSpriteObject : public BoundedObjectHeader
    {
      public:
        Transform matrix;
        char* getSprite()   {return sizeof(TransformedSpriteObject)+(char*)this;}
    };
};


#endif
