//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef GuiWimpWindow_h
#define GuiWimpWindow_h

class GuiWindowPositionBlock
{
  public:
    int      windowHandle;   
    GuiBBox  visibleArea;
    int      xscroll;
    int      yscroll;
    
    // conversion routines
    int      xToScreen(int x) const       {return x-xscroll+visibleArea.xmin;}
    int      yToScreen(int y) const       {return y-yscroll+visibleArea.ymax;}
    int      xToWorkarea(int x) const     {return x+xscroll-visibleArea.xmin;}
    int      yToWorkarea(int y) const     {return y+yscroll-visibleArea.ymax;}
    int      windowTopToWorkarea() const  {return yscroll;}
    int      windowLeftToWorkarea() const {return xscroll;}
    GuiBBox  toScreen(const GuiBBox &in) const;
    GuiBBox  toWorkarea(const GuiBBox &in) const;
};

class GuiRedrawWindowBlock : public GuiWindowPositionBlock
{
  public:
    GuiBBox redrawArea;
};

class GuiGetWindowStateBlock : public GuiWindowPositionBlock
{
  public:
    int          behind;
    unsigned int flags;
    int          parent;
    int          linkFlags;
};

typedef GuiGetWindowStateBlock  GuiOpenWindowBlock;

//**************************************************************************
//   inlines
//**************************************************************************

inline GuiBBox GuiWindowPositionBlock::toScreen(const GuiBBox &in) const
{
    GuiBBox out;
    out.xmin=xToScreen(in.xmin);
    out.xmax=xToScreen(in.xmax);
    out.ymin=yToScreen(in.ymin);
    out.ymax=yToScreen(in.ymax);
    return out;
}

inline GuiBBox GuiWindowPositionBlock::toWorkarea(const GuiBBox &in) const
{
    GuiBBox out;
    out.xmin=xToWorkarea(in.xmin);
    out.xmax=xToWorkarea(in.xmax);
    out.ymin=yToWorkarea(in.ymin);
    out.ymax=yToWorkarea(in.ymax);
    return out;
}

#endif
