//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above
//        copyright notice, this list of conditions and the following
//        disclaimer in the documentation and/or other materials
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote
//        products derived from this software without specific prior
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DrawOutputDevice_h
#define DrawOutputDevice_h

#include "Object.h"
#include "OutputDev.h"
#include "GuiDrawFile.h"
#include "GuiDrawFilePath.h"
#include "GuiDrawFileText.h"
#include "DrawOutputFont.h"
#include "Clip.h"

class DrawOutputDevice : public OutputDev
{
  public:
    DrawOutputDevice();
    virtual ~DrawOutputDevice();

    virtual GBool upsideDown();
    virtual GBool useDrawChar();
    virtual GBool interpretType3Chars();

    virtual void startPage(int pageNum, GfxState *state);
    virtual void endPage();

    //----- save/restore graphics state
    virtual void saveState(GfxState *state);
    virtual void restoreState(GfxState *state);

    //----- update graphics state
    virtual void updateLineJoin(GfxState*);
    virtual void updateLineCap(GfxState*);
    virtual void updateLineWidth(GfxState*);
    virtual void updateFillColor(GfxState*);
    virtual void updateStrokeColor(GfxState*state);

    //----- path painting
    virtual void stroke(GfxState*);
    virtual void fill(GfxState*);
    virtual void eoFill(GfxState*);

    //----- path clipping
    virtual void clip(GfxState*);
    virtual void eoClip(GfxState*);

    //----- text drawing

    virtual void updateFont(GfxState *state);
    virtual void updateRender(GfxState *state);

    virtual void drawChar(GfxState *state, double x, double y,
                        double dx, double dy,
                        double originX, double originY,
                        CharCode code, int nBytes, Unicode *u, int uLen);
    virtual void drawString(GfxState *state, GString *s);

    //----- image drawing
    virtual void drawImageMask(GfxState *state, Object *ref, Stream *str,
                             int width, int height, GBool invert,
                             GBool inlineImg);
    virtual void drawImage(GfxState *state, Object *ref, Stream *str,
                         int width, int height, GfxImageColorMap *colorMap,
                         int *maskColors, GBool inlineImg);


    GuiDrawFile& getDrawFile()                    {return draw;}

    void   setMoreColours(bool on)                {moreColours=on;}
    void   setNoImages(bool on)                   {noImages=on;}
    void   setNoText(bool on)                     {noText=on;}
    void   setNoType3Fonts(bool on)               {noType3Fonts=on;}
    void   setNoDrawings(bool on)                 {noDrawings=on;}

    enum {DPI = 180*256};
  private:
    GuiDrawFile          draw;
    GuiDrawFilePathState pathState;
    GuiDrawFilePath      path;
    GuiDrawFileText      text;
    DrawOutputFontList   font;
    Clip                 clipbox;
    GuiBBox              bounds;
    bool                 moreColours;
    bool                 noImages;
    bool                 noText;
    bool                 noType3Fonts;
    bool                 noDrawings;

    bool doPath(GfxState*);
    bool doRectanglePath(GfxState*);
    void setClip(GfxState*,int isEo);
};

#endif
