//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef PageHistory_h
#define PageHistory_h

#define PAGE_HISTORY_SIZE 32

#include "GuiTargets.h"
#include "GuiGadget.h"
#include "DocPage.h"

class DocView;
class DocViewToolbar;

class PageHistory
{
  public:

    PageHistory(DocView&, DocViewToolbar&,GuiWindow& ancestor);
    ~PageHistory();

    void add(int num);
    void addPending(int num);            
    void addPending();

    const DocPage*  back();
    const DocPage*  forward();

  private:
    DocView&        view;
    GuiToolAction   backIcon;
    GuiToolAction   forwardIcon;
    int             pendingPage;
    int             begin;
    int             end;
    int             current;

    DocPage         buf[PAGE_HISTORY_SIZE];
  
    void            setFade();
    void            setPage(const DocPage*);

    GUI_DECLARE_EVENT_TARGETS(PageHistory);

    GuiToolboxTarget adjusterClickedTarget;
    Claim adjusterClicked(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget historyBackTarget;
    Claim historyBack(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget historyForwardTarget;
    Claim historyForward(GuiToolboxEvent&,const GuiIdBlock&);
};

#endif
