//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include <string.h>
#include <fstream>
#include <stdlib.h>
#include "UnicodeToAcornMap.h"

#include <iostream>

UnicodeToAcornMap::UnicodeToAcornMap(const UnicodeToAcornMapData* data)
{
  memset((char*)map,NO_CONVERSION_CHAR,sizeof(map));

  for (int i=0;i>=0;i++)
  {
    unsigned int u = data[i].unicode;
    if (u==-1u) break;
    addChar(u,data[i].charnum);
  }
}


//*****************************************************

UnicodeToAcornMap::UnicodeToAcornMap()
{
  memset((char*)map,0,sizeof(map));
}


//*****************************************************

UnicodeToAcornMap::~UnicodeToAcornMap()
{
  int i;

  for (i=0;i<256;i++) {if (map[i]) delete map[i];}
}


//*****************************************************

void UnicodeToAcornMap::addChar(unsigned int u,unsigned char c)
{
  UnicodeToAcornLeaf* leaf=map[(u>>8) & 0xff];
  if (!leaf)
  {
    leaf=new UnicodeToAcornLeaf;
    if (!leaf) return;
    map[(u>>8) & 0xff]=leaf;
  }
  leaf->c[u & 0xff]=c;
}

//*****************************************************
//*****************************************************
//*****************************************************


UnicodeToAcornLeaf::UnicodeToAcornLeaf()
{
  memset((char*)c,0,sizeof(c));
}

//int main()
//{
//  UnicodeToAcornMap latin1("ADFS::HardDisc4.$.Projects.GuiLibProjects.Pdf.PdfNew.UnicodeToAcorn.AcornLat1");
//  UnicodeToAcornMap symbol("ADFS::HardDisc4.$.Projects.GuiLibProjects.Pdf.PdfNew.UnicodeToAcorn.symbol");
//  UnicodeToAcornMap dingbat("ADFS::HardDisc4.$.Projects.GuiLibProjects.Pdf.PdfNew.UnicodeToAcorn.zdingbat");
//  cout << hex << (int)dingbat.getChar(0x27be) << endl;
//}
